package com.study.libs;

import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.core.view.ViewCompat;

import com.study.libs.utils.SkinResources;
import com.study.libs.utils.SkinThemeUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 换肤属性相关
 * 这里面放了所有要换肤的view和其对应的属性
 */
public class SkinAttribute {

    private static final List<String> attributes = new ArrayList<>();

    static {
        attributes.add("background");
        attributes.add("src");
        attributes.add("textColor");
        attributes.add("drawableLeft");
        attributes.add("drawableTop");
        attributes.add("drawableRight");
        attributes.add("drawableBottom");
    }

    //记录换肤需要操作的View与属性信息
    private List<SkinView> skinViewList = new ArrayList<>();

    /**
     * 对一些view进行初始化换肤，用于xml解析过程中或者view创建过程中
     */
    public void look(View view, AttributeSet attrs) {
        List<SkinPair> skinPairs = new ArrayList<>();
        //读取xml中的该view的属性
        for (int i = 0; i < attrs.getAttributeCount(); i++) {
            String attributeName = attrs.getAttributeName(i);//这里获取的是R文件中的内容
            //如果属于我们预置的属性，比如要进行置换的background等
            if (attributes.contains(attributeName)) {
                //获取属性的值
                String attributeValue = attrs.getAttributeValue(i);
                if (attributeValue.startsWith("#")) {//写死的颜色，不用管
                    continue;
                }
                int resId = 0;
                if (attributeValue.startsWith("?")) {//以？开头的应该是系统theme，需要做一次转换获取属性
                    int attrId = Integer.parseInt(attributeValue.substring(1));//todo 这里的为何要转换成Integer
                    resId = SkinThemeUtils.getResId(view.getContext(), new int[]{attrId})[0];
                } else {//正常是以@开头，后面跟地址
                    resId = Integer.parseInt(attributeValue.substring(1));
                }
                if (resId != 0) {
                    SkinPair skinPair = new SkinPair(attributeName, resId);
                    skinPairs.add(skinPair);
                }
            }
        }
        if (!skinPairs.isEmpty() || view instanceof SkinViewSupport) {
            SkinView skinView = new SkinView(view, skinPairs);
            //view替换属性为skin包中的属性，即换肤
            skinView.applySkin();
            skinViewList.add(skinView);
        }
    }

    /**
     * 对所有的view进行更新。一般用于统一更新内容
     */
    public void applySkin() {
        for (SkinView skinView : skinViewList) {
            skinView.applySkin();
        }
    }

    /**
     * 移除所有view
     */
    public void clearAll() {
        skinViewList.clear();
    }

    static class SkinView {
        View view;
        //这个View的能被换肤的属性与它对应的id的集合
        List<SkinPair> skinPairList;

        public SkinView(View view, List<SkinPair> skinPairList) {
            this.view = view;
            this.skinPairList = skinPairList;
        }

        /**
         * 对一个View中的所有符合类型的属性进行修改
         */
        public void applySkin() {
            applySkinSupport();
            //遍历skinPair的列表，查看view的属性，对可操作的属性进行替换
            for (SkinPair skinPair : skinPairList) {
                if (skinPair.resId == 0) {//如果属性无效，就不
                    continue;
                }
                Drawable left = null, top = null, right = null, bottom = null;
                switch (skinPair.attributeName) {
                    case "background":
                        Object background = SkinResources.getInstance().getBackground(skinPair
                                .resId);
                        //背景可能是 @color 也可能是 @drawable
                        if (background instanceof Integer) {
                            view.setBackgroundColor((int) background);
                        } else {
                            ViewCompat.setBackground(view, (Drawable) background);
                        }
                        break;
                    case "src":
                        background = SkinResources.getInstance().getBackground(skinPair.resId);
                        if (background instanceof Integer) {
                            ((ImageView) view).setImageDrawable(new ColorDrawable((Integer)
                                    background));
                        } else {
                            ((ImageView) view).setImageDrawable((Drawable) background);
                        }
                        break;
                    case "textColor":
                        ((TextView) view).setTextColor(SkinResources.getInstance().getColorStateList
                                (skinPair.resId));
                        break;
                    case "drawableLeft":
                        left = SkinResources.getInstance().getDrawable(skinPair.resId);
                        break;
                    case "drawableTop":
                        top = SkinResources.getInstance().getDrawable(skinPair.resId);
                        break;
                    case "drawableRight":
                        right = SkinResources.getInstance().getDrawable(skinPair.resId);
                        break;
                    case "drawableBottom":
                        bottom = SkinResources.getInstance().getDrawable(skinPair.resId);
                        break;
                    default:
                        break;
                }
                if (null != left || null != right || null != top || null != bottom) {
                    ((TextView) view).setCompoundDrawablesWithIntrinsicBounds(left, top, right,
                            bottom);
                }
            }
        }

        /**
         * 自定义view如果继承了接口，就先调用接口中的applySkin
         * 提问：这里是不是调用接口中的applySkin之后，就不用调用默认的了？
         * 答：不是，这里是允许对自定义view的自定义属性进行皮肤更改，没有办法只能有一些侵入性。在这之后才对默认的一些属性进行更改
         */
        private void applySkinSupport() {
            if (view instanceof SkinViewSupport) {
                ((SkinViewSupport) view).applySkin();
            }
        }
    }

    static class SkinPair {
        //属性名称
        String attributeName;
        //对应的资源id
        int resId;

        public SkinPair(String attributeName, int resId) {
            this.attributeName = attributeName;
            this.resId = resId;
        }
    }

}
