package com.study.libs;

import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import com.study.libs.utils.SkinThemeUtils;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;

/**
 * 用来接管系统view的生产过程
 */
public class SkinLayoutInflaterFactory implements LayoutInflater.Factory2, Observer {
    //记录对应view的构造函数。每个构造方法都用两参的
    private static final Class<?>[] mConstructorSignature = new Class[]{
            Context.class, AttributeSet.class};

    private static final HashMap<String, Constructor<? extends View>> sConstructorMap =
            new HashMap<>();

    private static final String[] mClassPrefixList = {
            "android.widget.",
            "android.webkit.",
            "android.app.",
            "android.view."
    };

    //属性管理器
    private SkinAttribute skinAttribute;

    private Activity activity;

    public SkinLayoutInflaterFactory(Activity activity) {
        this.activity = activity;
        skinAttribute = new SkinAttribute();
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Nullable
    @Override
    public View onCreateView(@Nullable View parent, @NonNull String name, @NonNull Context context, @NonNull AttributeSet attrs) {
        //优先创建系统自带的view
        View view = createSDKView(name, context, attrs);
        if (view == null) {
            view = createView(name, context, attrs);
        }
        //只要创建成功，就记录其参数
        if (null != view) {
            skinAttribute.look(view, attrs);
        }
        return view;
    }


    /**
     * 创建系统自带的view
     */
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private View createSDKView(String name, Context context, AttributeSet attributeSet) {
        //如果xml中包含点，则可能是自定义的view，这里不对其进行处理
        if (name.contains(".")) {//如果不包含点
            return null;
        }
        for (int i = 0; i < mClassPrefixList.length; i++) {
            //尝试添加前缀，进行反射
            View view = createView(mClassPrefixList[i] + name, context, attributeSet);
            if (view != null) {//直到反射出来，返回
                return view;
            }
        }
        return null;
    }

    /**
     * 创建view
     */
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private View createView(@NonNull String name, @NonNull Context context, @NonNull AttributeSet attrs) {
        Constructor<? extends View> constructor = findConstructor(context, name);
        try {
            //创建实例
            return constructor.newInstance(context, attrs);
        } catch (Exception e) {
            //不必输出
//            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取类的构造函数,并存起来后面继续用
     */
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private Constructor<? extends View> findConstructor(Context context, String name) {
        Constructor<? extends View> constructor = sConstructorMap.get(name);
        if (constructor == null) {
            try {
                Class<? extends View> clazz =
                        context.getClassLoader().loadClass(name).asSubclass(View.class);//获取一个view的class
                constructor = clazz.getConstructor(mConstructorSignature);
                constructor.setAccessible(true);
                sConstructorMap.put(name, constructor);
            } catch (ClassNotFoundException | NoSuchMethodException e) {
//                e.printStackTrace();
            }
        }
        return constructor;
    }


    /**
     * 这个基本不用
     */
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Nullable
    @Override
    public View onCreateView(@NonNull String name, @NonNull Context context, @NonNull AttributeSet attrs) {
        return onCreateView(null, name, context, attrs);
    }


    /**
     * 观察者的执行代码
     * 这里在运行时换肤
     */
    @Override
    public void update(Observable o, Object arg) {
        if (activity != null) {
            SkinThemeUtils.updateStatusBarColor(activity);
        }
        if (skinAttribute != null) {
            skinAttribute.applySkin();
        }
    }

    /**
     * 清理
     */
    public void release() {
        activity = null;
        if (skinAttribute != null) {
            skinAttribute.clearAll();
        }
    }
}
